@@ -5,7 +5,8 @@
 # Space between two letters should be one DAH.
 # Space between two words should be DOT DAH DAH.
 
-import sys, math, audiodev
+import sys, math, aifc
+from contextlib import closing
 
 DOT = 30
 DAH = 3 * DOT
@@ -51,18 +52,18 @@ morsetab = {
         ' ': ' ',               '_': '..--.-',
 }
 
-nowave = '\0' * 200
+nowave = b'\0' * 200
 
 # If we play at 44.1 kHz (which we do), then if we produce one sine
 # wave in 100 samples, we get a tone of 441 Hz.  If we produce two
 # sine waves in these 100 samples, we get a tone of 882 Hz.  882 Hz
 # appears to be a nice one for playing morse code.
 def mkwave(octave):
-    sinewave = ''
+    sinewave = bytearray()
     for i in range(100):
         val = int(math.sin(math.pi * i * octave / 50.0) * 30000)
-        sinewave += chr((val >> 8) & 255) + chr(val & 255)
-    return sinewave
+        sinewave.extend([(val >> 8) & 255, val & 255])
+    return bytes(sinewave)
 
 defaultwave = mkwave(OCTAVE)
 
@@ -74,35 +75,24 @@ def main():
         sys.stderr.write('Usage ' + sys.argv[0] +
                          ' [ -o outfile ] [ -p octave ] [ words ] ...\n')
         sys.exit(1)
-    dev = None
     wave = defaultwave
+    outfile = 'morse.aifc'
     for o, a in opts:
         if o == '-o':
-            import aifc
-            dev = aifc.open(a, 'w')
-            dev.setframerate(44100)
-            dev.setsampwidth(2)
-            dev.setnchannels(1)
+            outfile = a
         if o == '-p':
             wave = mkwave(int(a))
-    if not dev:
-        import audiodev
-        dev = audiodev.AudioDev()
-        dev.setoutrate(44100)
-        dev.setsampwidth(2)
-        dev.setnchannels(1)
-        dev.close = dev.stop
-        dev.writeframesraw = dev.writeframes
-    if args:
-        source = [' '.join(args)]
-    else:
-        source = iter(sys.stdin.readline, '')
-    for line in source:
-        mline = morse(line)
-        play(mline, dev, wave)
-        if hasattr(dev, 'wait'):
-            dev.wait()
-    dev.close()
+    with closing(aifc.open(outfile, 'w')) as fp:
+        fp.setframerate(44100)
+        fp.setsampwidth(2)
+        fp.setnchannels(1)
+        if args:
+            source = [' '.join(args)]
+        else:
+            source = iter(sys.stdin.readline, '')
+        for line in source:
+            mline = morse(line)
+            play(mline, fp, wave)
 
 # Convert a string to morse code with \001 between the characters in
 # the string.
@@ -116,23 +106,23 @@ def morse(line):
     return res
 
 # Play a line of morse code.
-def play(line, dev, wave):
+def play(line, fp, wave):
     for c in line:
         if c == '.':
-            sine(dev, DOT, wave)
+            sine(fp, DOT, wave)
         elif c == '-':
-            sine(dev, DAH, wave)
+            sine(fp, DAH, wave)
         else:                   # space
-            pause(dev, DAH + DOT)
-        pause(dev, DOT)
+            pause(fp, DAH + DOT)
+        pause(fp, DOT)
 
-def sine(dev, length, wave):
+def sine(fp, length, wave):
     for i in range(length):
-        dev.writeframesraw(wave)
+        fp.writeframesraw(wave)
 
-def pause(dev, length):
+def pause(fp, length):
     for i in range(length):
-        dev.writeframesraw(nowave)
+        fp.writeframesraw(nowave)
 
 if __name__ == '__main__':
     main()
