@@ -1,10 +1,12 @@
 # Echo client demo using Unix sockets
 # Piet van Oostrum
+
 from socket import *
-FILE = 'blabla'
+
+FILE = 'unix-socket'
 s = socket(AF_UNIX, SOCK_STREAM)
 s.connect(FILE)
-s.send('Hello, world')
+s.send(b'Hello, world')
 data = s.recv(1024)
 s.close()
-print 'Received', `data`
+print('Received', repr(data))
