@@ -2,7 +2,7 @@
 
 """Play with the new Tk 8.0 toplevel menu option."""
 
-from Tkinter import *
+from tkinter import *
 
 class App:
 
