@@ -1,10 +1,10 @@
-from Tkinter import *
+from tkinter import *
 
 # this shows how to spawn off new windows at a button press
 
 class Test(Frame):
     def printit(self):
-        print "hi"
+        print("hi")
 
     def makeWindow(self):
         fred = Toplevel()
