@@ -1,4 +1,4 @@
-#!/usr/bin/env python
+#!/usr/bin/python
 """       turtle-example-suite:
 
         tdemo_planets_and_moon.py
@@ -12,9 +12,9 @@ very light moon!
 Planet has a circular orbit, moon a stable
 orbit around the planet.
 
-You can hold the movement temporarily by
-pressing the left mouse button with the
-mouse over the scrollbar of the canvas.
+You can hold the movement temporarily by pressing
+the left mouse button with mouse over the
+scrollbar of the canvas.
 
 """
 from turtle import Shape, Turtle, mainloop, Vec2D as Vec
@@ -71,7 +71,7 @@ class Star(Turtle):
 def main():
     s = Turtle()
     s.reset()
-    s.tracer(0,0)
+    s.getscreen().tracer(0,0)
     s.ht()
     s.pu()
     s.fd(6)
@@ -89,7 +89,7 @@ def main():
     planetshape.addcomponent(m1,"orange")
     planetshape.addcomponent(m2,"blue")
     s.getscreen().register_shape("planet", planetshape)
-    s.tracer(1,0)
+    s.getscreen().tracer(1,0)
 
     ## setup gravitational system
     gs = GravSys()
@@ -108,5 +108,6 @@ def main():
     return "Done!"
 
 if __name__ == '__main__':
-    main()
-    mainloop()
+    msg = main()
+    print(msg)
+    #mainloop()
