@@ -473,42 +473,42 @@ class DictTest(unittest.TestCase):
         k2 = {1:1, 2:2, 3:3}.keys()
         k3 = {4:4}.keys()
 
-        self.assertEquals(k1 - k2, set())
-        self.assertEquals(k1 - k3, {1,2})
-        self.assertEquals(k2 - k1, {3})
-        self.assertEquals(k3 - k1, {4})
-        self.assertEquals(k1 & k2, {1,2})
-        self.assertEquals(k1 & k3, set())
-        self.assertEquals(k1 | k2, {1,2,3})
-        self.assertEquals(k1 ^ k2, {3})
-        self.assertEquals(k1 ^ k3, {1,2,4})
+        self.assertEqual(k1 - k2, set())
+        self.assertEqual(k1 - k3, {1,2})
+        self.assertEqual(k2 - k1, {3})
+        self.assertEqual(k3 - k1, {4})
+        self.assertEqual(k1 & k2, {1,2})
+        self.assertEqual(k1 & k3, set())
+        self.assertEqual(k1 | k2, {1,2,3})
+        self.assertEqual(k1 ^ k2, {3})
+        self.assertEqual(k1 ^ k3, {1,2,4})
 
     def test_dictview_set_operations_on_items(self):
         k1 = {1:1, 2:2}.items()
         k2 = {1:1, 2:2, 3:3}.items()
         k3 = {4:4}.items()
 
-        self.assertEquals(k1 - k2, set())
-        self.assertEquals(k1 - k3, {(1,1), (2,2)})
-        self.assertEquals(k2 - k1, {(3,3)})
-        self.assertEquals(k3 - k1, {(4,4)})
-        self.assertEquals(k1 & k2, {(1,1), (2,2)})
-        self.assertEquals(k1 & k3, set())
-        self.assertEquals(k1 | k2, {(1,1), (2,2), (3,3)})
-        self.assertEquals(k1 ^ k2, {(3,3)})
-        self.assertEquals(k1 ^ k3, {(1,1), (2,2), (4,4)})
+        self.assertEqual(k1 - k2, set())
+        self.assertEqual(k1 - k3, {(1,1), (2,2)})
+        self.assertEqual(k2 - k1, {(3,3)})
+        self.assertEqual(k3 - k1, {(4,4)})
+        self.assertEqual(k1 & k2, {(1,1), (2,2)})
+        self.assertEqual(k1 & k3, set())
+        self.assertEqual(k1 | k2, {(1,1), (2,2), (3,3)})
+        self.assertEqual(k1 ^ k2, {(3,3)})
+        self.assertEqual(k1 ^ k3, {(1,1), (2,2), (4,4)})
 
     def test_dictview_mixed_set_operations(self):
         # Just a few for .keys()
         self.assertTrue({1:1}.keys() == {1})
         self.assertTrue({1} == {1:1}.keys())
-        self.assertEquals({1:1}.keys() | {2}, {1, 2})
-        self.assertEquals({2} | {1:1}.keys(), {1, 2})
+        self.assertEqual({1:1}.keys() | {2}, {1, 2})
+        self.assertEqual({2} | {1:1}.keys(), {1, 2})
         # And a few for .items()
         self.assertTrue({1:1}.items() == {(1,1)})
         self.assertTrue({(1,1)} == {1:1}.items())
-        self.assertEquals({1:1}.items() | {2}, {(1,1), 2})
-        self.assertEquals({2} | {1:1}.items(), {(1,1), 2})
+        self.assertEqual({1:1}.items() | {2}, {(1,1), 2})
+        self.assertEqual({2} | {1:1}.items(), {(1,1), 2})
 
     def test_missing(self):
         # Make sure dict doesn't have a __missing__ method
