@@ -176,6 +176,9 @@ PyObject *PyExc_BlockingIOError = (PyObject *)&_PyExc_BlockingIOError;
  * The main open() function
  */
 PyDoc_STRVAR(open_doc,
+"open(file, mode='r', buffering=-1, encoding=None,\n"
+"     errors=None, newline=None, closefd=True) -> file object\n"
+"\n"
 "Open file and return a stream.  Raise IOError upon failure.\n"
 "\n"
 "file is either a text or byte string giving the name (and the path\n"
@@ -448,7 +451,7 @@ io_open(PyObject *self, PyObject *args, PyObject *kwds)
             if (fileno == -1 && PyErr_Occurred())
                 goto error;
 
-            if (fstat(fileno, &st) >= 0)
+            if (fstat(fileno, &st) >= 0 && st.st_blksize > 1)
                 buffering = st.st_blksize;
         }
 #endif
