@@ -3,14 +3,15 @@
 """ A Python Benchmark Suite
 
 """
-#
-# Note: Please keep this module compatible to Python 1.5.2.
+# Note: Please keep this module compatible to Python 2.6.
 #
 # Tests may include features in later Python versions, but these
 # should then be embedded in try-except clauses in the configuration
 # module Setup.py.
 #
 
+from __future__ import print_function
+
 # pybench Copyright
 __copyright__ = """\
 Copyright (c), 1997-2006, Marc-Andre Lemburg (mal@lemburg.com)
@@ -34,7 +35,7 @@ NEGLIGENCE OR OTHER TORTIOUS ACTION, ARISING OUT OF OR IN CONNECTION
 WITH THE USE OR PERFORMANCE OF THIS SOFTWARE !
 """
 
-import sys, time, operator, string, platform
+import sys, time, operator, platform
 from CommandLine import *
 
 try:
@@ -44,7 +45,7 @@ except ImportError:
     import pickle
 
 # Version number; version history: see README file !
-__version__ = '2.0'
+__version__ = '2.1'
 
 ### Constants
 
@@ -103,19 +104,15 @@ def get_timer(timertype):
 def get_machine_details():
 
     if _debug:
-        print 'Getting machine details...'
+        print('Getting machine details...')
     buildno, builddate = platform.python_build()
     python = platform.python_version()
-    try:
-        unichr(100000)
-    except ValueError:
+    if sys.maxunicode == 65535:
         # UCS2 build (standard)
-        unicode = 'UCS2'
-    except NameError:
-        unicode = None
+        unitype = 'UCS2'
     else:
         # UCS4 build (most recent Linux distros)
-        unicode = 'UCS4'
+        unitype = 'UCS4'
     bits, linkage = platform.architecture()
     return {
         'platform': platform.platform(),
@@ -127,7 +124,7 @@ def get_machine_details():
         'compiler': platform.python_compiler(),
         'buildno': buildno,
         'builddate': builddate,
-        'unicode': unicode,
+        'unicode': unitype,
         'bits': bits,
         }
 
@@ -147,7 +144,8 @@ def print_machine_details(d, indent=''):
                                           d.get('buildno', 'n/a')),
          '   Unicode:        %s' % d.get('unicode', 'n/a'),
          ]
-    print indent + string.join(l, '\n' + indent) + '\n'
+    joiner = '\n' + indent
+    print(indent + joiner.join(l) + '\n')
 
 ### Test baseclass
 
@@ -177,7 +175,7 @@ class Test:
     # Version number of the test as float (x.yy); this is important
     # for comparisons of benchmark runs - tests with unequal version
     # number will not get compared.
-    version = 2.0
+    version = 2.1
 
     # The number of abstract operations done in each round of the
     # test. An operation is the basic unit of what you want to
@@ -281,9 +279,9 @@ class Test:
             prep_times.append(t)
         min_prep_time = min(prep_times)
         if _debug:
-            print
-            print 'Calib. prep time     = %.6fms' % (
-                min_prep_time * MILLI_SECONDS)
+            print()
+            print('Calib. prep time     = %.6fms' % (
+                min_prep_time * MILLI_SECONDS))
 
         # Time the calibration runs (doing CALIBRATION_LOOPS loops of
         # .calibrate() method calls each)
@@ -299,8 +297,8 @@ class Test:
         min_overhead = min(self.overhead_times)
         max_overhead = max(self.overhead_times)
         if _debug:
-            print 'Calib. overhead time = %.6fms' % (
-                min_overhead * MILLI_SECONDS)
+            print('Calib. overhead time = %.6fms' % (
+                min_overhead * MILLI_SECONDS))
         if min_overhead < 0.0:
             raise ValueError('calibration setup did not work')
         if max_overhead - min_overhead > 0.1:
@@ -371,7 +369,7 @@ class Test:
         if runs == 0:
             return 0.0, 0.0, 0.0, 0.0
         min_time = min(self.times)
-        total_time = reduce(operator.add, self.times, 0.0)
+        total_time = sum(self.times)
         avg_time = total_time / float(runs)
         operation_avg = total_time / float(runs
                                            * self.rounds
@@ -406,7 +404,7 @@ class Benchmark:
     roundtime = 0
 
     # Benchmark version number as float x.yy
-    version = 2.0
+    version = 2.1
 
     # Produce verbose output ?
     verbose = 0
@@ -437,7 +435,7 @@ class Benchmark:
         # Init vars
         self.tests = {}
         if _debug:
-            print 'Getting machine details...'
+            print('Getting machine details...')
         self.machine_details = get_machine_details()
 
         # Make .version an instance attribute to have it saved in the
@@ -474,8 +472,8 @@ class Benchmark:
 
         # Add tests
         if self.verbose:
-            print 'Searching for tests ...'
-            print '--------------------------------------'
+            print('Searching for tests ...')
+            print('--------------------------------------')
         for testclass in setupmod.__dict__.values():
             if not hasattr(testclass, 'is_a_test'):
                 continue
@@ -489,78 +487,75 @@ class Benchmark:
                 warp=self.warp,
                 calibration_runs=self.calibration_runs,
                 timer=self.timer)
-        l = self.tests.keys()
-        l.sort()
+        l = sorted(self.tests)
         if self.verbose:
             for name in l:
-                print '  %s' % name
-            print '--------------------------------------'
-            print '  %i tests found' % len(l)
-            print
+                print('  %s' % name)
+            print('--------------------------------------')
+            print('  %i tests found' % len(l))
+            print()
 
     def calibrate(self):
 
-        print 'Calibrating tests. Please wait...',
+        print('Calibrating tests. Please wait...', end=' ')
         sys.stdout.flush()
         if self.verbose:
-            print
-            print
-            print 'Test                              min      max'
-            print '-' * LINE
-        tests = self.tests.items()
-        tests.sort()
+            print()
+            print()
+            print('Test                              min      max')
+            print('-' * LINE)
+        tests = sorted(self.tests.items())
         for i in range(len(tests)):
             name, test = tests[i]
             test.calibrate_test()
             if self.verbose:
-                print '%30s:  %6.3fms  %6.3fms' % \
+                print('%30s:  %6.3fms  %6.3fms' % \
                       (name,
                        min(test.overhead_times) * MILLI_SECONDS,
-                       max(test.overhead_times) * MILLI_SECONDS)
+                       max(test.overhead_times) * MILLI_SECONDS))
         if self.verbose:
-            print
-            print 'Done with the calibration.'
+            print()
+            print('Done with the calibration.')
         else:
-            print 'done.'
-        print
+            print('done.')
+        print()
 
     def run(self):
 
-        tests = self.tests.items()
-        tests.sort()
+        tests = sorted(self.tests.items())
         timer = self.get_timer()
-        print 'Running %i round(s) of the suite at warp factor %i:' % \
-              (self.rounds, self.warp)
-        print
+        print('Running %i round(s) of the suite at warp factor %i:' % \
+              (self.rounds, self.warp))
+        print()
         self.roundtimes = []
         for i in range(self.rounds):
             if self.verbose:
-                print ' Round %-25i  effective   absolute  overhead' % (i+1)
+                print(' Round %-25i  effective   absolute  overhead' % (i+1))
             total_eff_time = 0.0
             for j in range(len(tests)):
                 name, test = tests[j]
                 if self.verbose:
-                    print '%30s:' % name,
+                    print('%30s:' % name, end=' ')
                 test.run()
                 (eff_time, abs_time, min_overhead) = test.last_timing
                 total_eff_time = total_eff_time + eff_time
                 if self.verbose:
-                    print '    %5.0fms    %5.0fms %7.3fms' % \
+                    print('    %5.0fms    %5.0fms %7.3fms' % \
                           (eff_time * MILLI_SECONDS,
                            abs_time * MILLI_SECONDS,
-                           min_overhead * MILLI_SECONDS)
+                           min_overhead * MILLI_SECONDS))
             self.roundtimes.append(total_eff_time)
             if self.verbose:
-                print ('                   '
+                print('                   '
                        '               ------------------------------')
-                print ('                   '
+                print('                   '
                        '     Totals:    %6.0fms' %
                        (total_eff_time * MILLI_SECONDS))
-                print
+                print()
             else:
-                print '* Round %i done in %.3f seconds.' % (i+1,
-                                                            total_eff_time)
-        print
+                print('* Round %i done in %.3f seconds.' % (i+1,
+                                                            total_eff_time))
+        print()
 
     def stat(self):
 
@@ -578,32 +573,31 @@ class Benchmark:
         if runs == 0:
             return 0.0, 0.0
         min_time = min(self.roundtimes)
-        total_time = reduce(operator.add, self.roundtimes, 0.0)
+        total_time = sum(self.roundtimes)
         avg_time = total_time / float(runs)
         max_time = max(self.roundtimes)
         return (min_time, avg_time, max_time)
 
     def print_header(self, title='Benchmark'):
 
-        print '-' * LINE
-        print '%s: %s' % (title, self.name)
-        print '-' * LINE
-        print
-        print '    Rounds: %s' % self.rounds
-        print '    Warp:   %s' % self.warp
-        print '    Timer:  %s' % self.timer
-        print
+        print('-' * LINE)
+        print('%s: %s' % (title, self.name))
+        print('-' * LINE)
+        print()
+        print('    Rounds: %s' % self.rounds)
+        print('    Warp:   %s' % self.warp)
+        print('    Timer:  %s' % self.timer)
+        print()
         if self.machine_details:
             print_machine_details(self.machine_details, indent='    ')
-            print
+            print()
 
     def print_benchmark(self, hidenoise=0, limitnames=None):
 
-        print ('Test                          '
+        print('Test                          '
                '   minimum  average  operation  overhead')
-        print '-' * LINE
-        tests = self.tests.items()
-        tests.sort()
+        print('-' * LINE)
+        tests = sorted(self.tests.items())
         total_min_time = 0.0
         total_avg_time = 0.0
         for name, test in tests:
@@ -617,43 +611,42 @@ class Benchmark:
              min_overhead) = test.stat()
             total_min_time = total_min_time + min_time
             total_avg_time = total_avg_time + avg_time
-            print '%30s:  %5.0fms  %5.0fms  %6.2fus  %7.3fms' % \
+            print('%30s:  %5.0fms  %5.0fms  %6.2fus  %7.3fms' % \
                   (name,
                    min_time * MILLI_SECONDS,
                    avg_time * MILLI_SECONDS,
                    op_avg * MICRO_SECONDS,
-                   min_overhead *MILLI_SECONDS)
-        print '-' * LINE
-        print ('Totals:                        '
+                   min_overhead *MILLI_SECONDS))
+        print('-' * LINE)
+        print('Totals:                        '
                ' %6.0fms %6.0fms' %
                (total_min_time * MILLI_SECONDS,
                 total_avg_time * MILLI_SECONDS,
                 ))
-        print
+        print()
 
     def print_comparison(self, compare_to, hidenoise=0, limitnames=None):
 
         # Check benchmark versions
         if compare_to.version != self.version:
-            print ('* Benchmark versions differ: '
+            print('* Benchmark versions differ: '
                    'cannot compare this benchmark to "%s" !' %
                    compare_to.name)
-            print
+            print()
             self.print_benchmark(hidenoise=hidenoise,
                                  limitnames=limitnames)
             return
 
         # Print header
         compare_to.print_header('Comparing with')
-        print ('Test                          '
+        print('Test                          '
                '   minimum run-time        average  run-time')
-        print ('                              '
+        print('                              '
                '   this    other   diff    this    other   diff')
-        print '-' * LINE
+        print('-' * LINE)
 
         # Print test comparisons
-        tests = self.tests.items()
-        tests.sort()
+        tests = sorted(self.tests.items())
         total_min_time = other_total_min_time = 0.0
         total_avg_time = other_total_avg_time = 0.0
         benchmarks_compatible = self.compatible(compare_to)
@@ -706,15 +699,15 @@ class Benchmark:
                     # Benchmark or tests are not comparible
                     min_diff, avg_diff = 'n/a', 'n/a'
                     tests_compatible = 0
-            print '%30s: %5.0fms %5.0fms %7s %5.0fms %5.0fms %7s' % \
+            print('%30s: %5.0fms %5.0fms %7s %5.0fms %5.0fms %7s' % \
                   (name,
                    min_time * MILLI_SECONDS,
                    other_min_time * MILLI_SECONDS * compare_to.warp / self.warp,
                    min_diff,
                    avg_time * MILLI_SECONDS,
                    other_avg_time * MILLI_SECONDS * compare_to.warp / self.warp,
-                   avg_diff)
-        print '-' * LINE
+                   avg_diff))
+        print('-' * LINE)
 
         # Summarise test results
         if not benchmarks_compatible or not tests_compatible:
@@ -732,7 +725,7 @@ class Benchmark:
                      (other_total_avg_time * compare_to.warp) - 1.0) * PERCENT)
             else:
                 avg_diff = 'n/a'
-        print ('Totals:                       '
+        print('Totals:                       '
                '  %5.0fms %5.0fms %7s %5.0fms %5.0fms %7s' %
                (total_min_time * MILLI_SECONDS,
                 (other_total_min_time * compare_to.warp/self.warp
@@ -743,10 +736,10 @@ class Benchmark:
                  * MILLI_SECONDS),
                 avg_diff
                ))
-        print
-        print '(this=%s, other=%s)' % (self.name,
-                                       compare_to.name)
-        print
+        print()
+        print('(this=%s, other=%s)' % (self.name,
+                                       compare_to.name))
+        print()
 
 class PyBenchCmdline(Application):
 
@@ -825,8 +818,8 @@ python pybench.py -s p25.pybench -c p21.pybench
         limitnames = self.values['-t']
         if limitnames:
             if _debug:
-                print '* limiting test names to one with substring "%s"' % \
-                      limitnames
+                print('* limiting test names to one with substring "%s"' % \
+                      limitnames)
             limitnames = re.compile(limitnames, re.I)
         else:
             limitnames = None
@@ -835,26 +828,26 @@ python pybench.py -s p25.pybench -c p21.pybench
         calibration_runs = self.values['-C']
         timer = self.values['--timer']
 
-        print '-' * LINE
-        print 'PYBENCH %s' % __version__
-        print '-' * LINE
-        print '* using %s %s' % (
+        print('-' * LINE)
+        print('PYBENCH %s' % __version__)
+        print('-' * LINE)
+        print('* using %s %s' % (
             getattr(platform, 'python_implementation', lambda:'Python')(),
-            string.join(string.split(sys.version), ' '))
+            ' '.join(sys.version.split())))
 
         # Switch off garbage collection
         if not withgc:
             try:
                 import gc
             except ImportError:
-                print '* Python version doesn\'t support garbage collection'
+                print('* Python version doesn\'t support garbage collection')
             else:
                 try:
                     gc.disable()
                 except NotImplementedError:
-                    print '* Python version doesn\'t support gc.disable'
+                    print('* Python version doesn\'t support gc.disable')
                 else:
-                    print '* disabled garbage collection'
+                    print('* disabled garbage collection')
 
         # "Disable" sys check interval
         if not withsyscheck:
@@ -863,18 +856,18 @@ python pybench.py -s p25.pybench -c p21.pybench
             try:
                 sys.setcheckinterval(value)
             except (AttributeError, NotImplementedError):
-                print '* Python version doesn\'t support sys.setcheckinterval'
+                print('* Python version doesn\'t support sys.setcheckinterval')
             else:
-                print '* system check interval set to maximum: %s' % value
+                print('* system check interval set to maximum: %s' % value)
 
         if timer == TIMER_SYSTIMES_PROCESSTIME:
             import systimes
-            print '* using timer: systimes.processtime (%s)' % \
-                  systimes.SYSTIMES_IMPLEMENTATION
+            print('* using timer: systimes.processtime (%s)' % \
+                  systimes.SYSTIMES_IMPLEMENTATION)
         else:
-            print '* using timer: %s' % timer
+            print('* using timer: %s' % timer)
 
-        print
+        print()
 
         if compare_to:
             try:
@@ -883,10 +876,10 @@ python pybench.py -s p25.pybench -c p21.pybench
                 bench.name = compare_to
                 f.close()
                 compare_to = bench
-            except IOError, reason:
-                print '* Error opening/reading file %s: %s' % (
+            except IOError as reason:
+                print('* Error opening/reading file %s: %s' % (
                     repr(compare_to),
-                    reason)
+                    reason))
                 compare_to = None
 
         if show_bench:
@@ -903,17 +896,17 @@ python pybench.py -s p25.pybench -c p21.pybench
                 else:
                     bench.print_benchmark(hidenoise=hidenoise,
                                           limitnames=limitnames)
-            except IOError, reason:
-                print '* Error opening/reading file %s: %s' % (
+            except IOError as reason:
+                print('* Error opening/reading file %s: %s' % (
                     repr(show_bench),
-                    reason)
-                print
+                    reason))
+                print()
             return
 
         if reportfile:
-            print 'Creating benchmark: %s (rounds=%i, warp=%i)' % \
-                  (reportfile, rounds, warp)
-            print
+            print('Creating benchmark: %s (rounds=%i, warp=%i)' % \
+                  (reportfile, rounds, warp))
+            print()
 
         # Create benchmark object
         bench = Benchmark(reportfile,
@@ -927,9 +920,9 @@ python pybench.py -s p25.pybench -c p21.pybench
             bench.calibrate()
             bench.run()
         except KeyboardInterrupt:
-            print
-            print '*** KeyboardInterrupt -- Aborting'
-            print
+            print()
+            print('*** KeyboardInterrupt -- Aborting')
+            print()
             return
         bench.print_header()
         if compare_to:
@@ -949,13 +942,13 @@ python pybench.py -s p25.pybench -c p21.pybench
                 bench.name = reportfile
                 pickle.dump(bench,f)
                 f.close()
-            except IOError, reason:
-                print '* Error opening/writing reportfile'
-            except IOError, reason:
-                print '* Error opening/writing reportfile %s: %s' % (
+            except IOError as reason:
+                print('* Error opening/writing reportfile')
+            except IOError as reason:
+                print('* Error opening/writing reportfile %s: %s' % (
                     reportfile,
-                    reason)
-                print
+                    reason))
+                print()
 
 if __name__ == '__main__':
     PyBenchCmdline()
