@@ -30,7 +30,7 @@
 # into a program for a different change to Python programs...
 
 import sys
-import regex
+import re
 import os
 from stat import *
 
@@ -53,17 +53,17 @@ def main():
             if fix(arg): bad = 1
     sys.exit(bad)
 
-ispythonprog = regex.compile('^[a-zA-Z0-9_]+\.py$')
+ispythonprog = re.compile('^[a-zA-Z0-9_]+\.py$')
 def ispython(name):
     return ispythonprog.match(name) >= 0
 
 def recursedown(dirname):
-    dbg('recursedown(' + `dirname` + ')\n')
+    dbg('recursedown(%r)\n' % (dirname,))
     bad = 0
     try:
         names = os.listdir(dirname)
-    except os.error, msg:
-        err(dirname + ': cannot list directory: ' + `msg` + '\n')
+    except os.error as msg:
+        err('%s: cannot list directory: %r\n' % (dirname, msg))
         return 1
     names.sort()
     subdirs = []
@@ -80,11 +80,11 @@ def recursedown(dirname):
     return bad
 
 def fix(filename):
-##  dbg('fix(' + `filename` + ')\n')
+##  dbg('fix(%r)\n' % (filename,))
     try:
         f = open(filename, 'r')
-    except IOError, msg:
-        err(filename + ': cannot open: ' + `msg` + '\n')
+    except IOError as msg:
+        err('%s: cannot open: %r\n' % (filename, msg))
         return 1
     head, tail = os.path.split(filename)
     tempname = os.path.join(head, '@' + tail)
@@ -106,16 +106,15 @@ def fix(filename):
             if g is None:
                 try:
                     g = open(tempname, 'w')
-                except IOError, msg:
+                except IOError as msg:
                     f.close()
-                    err(tempname+': cannot create: '+\
-                        `msg`+'\n')
+                    err('%s: cannot create: %r\n' % (tempname, msg))
                     return 1
                 f.seek(0)
                 lineno = 0
                 rep(filename + ':\n')
                 continue # restart from the beginning
-            rep(`lineno` + '\n')
+            rep(repr(lineno) + '\n')
             rep('< ' + line)
             rep('> ' + newline)
         if g is not None:
@@ -130,33 +129,31 @@ def fix(filename):
     # First copy the file's mode to the temp file
     try:
         statbuf = os.stat(filename)
-        os.chmod(tempname, statbuf[ST_MODE] & 07777)
-    except os.error, msg:
-        err(tempname + ': warning: chmod failed (' + `msg` + ')\n')
+        os.chmod(tempname, statbuf[ST_MODE] & 0o7777)
+    except os.error as msg:
+        err('%s: warning: chmod failed (%r)\n' % (tempname, msg))
     # Then make a backup of the original file as filename~
     try:
         os.rename(filename, filename + '~')
-    except os.error, msg:
-        err(filename + ': warning: backup failed (' + `msg` + ')\n')
+    except os.error as msg:
+        err('%s: warning: backup failed (%r)\n' % (filename, msg))
     # Now move the temp file to the original file
     try:
         os.rename(tempname, filename)
-    except os.error, msg:
-        err(filename + ': rename failed (' + `msg` + ')\n')
+    except os.error as msg:
+        err('%s: rename failed (%r)\n' % (filename, msg))
         return 1
     # Return succes
     return 0
 
 # This expression doesn't catch *all* class definition headers,
 # but it's pretty darn close.
-classexpr = '^\([ \t]*class +[a-zA-Z0-9_]+\) *( *) *\(\(=.*\)?\):'
-classprog = regex.compile(classexpr)
+classexpr = '^([ \t]*class +[a-zA-Z0-9_]+) *( *) *((=.*)?):'
+classprog = re.compile(classexpr)
 
 # Expressions for finding base class expressions.
-baseexpr = '^ *\(.*\) *( *) *$'
-baseprog = regex.compile(baseexpr)
-
-import string
+baseexpr = '^ *(.*) *( *) *$'
+baseprog = re.compile(baseexpr)
 
 def fixline(line):
     if classprog.match(line) < 0: # No 'class' keyword -- no change
@@ -176,7 +173,7 @@ def fixline(line):
     basepart = line[a2+1:b2]
 
     # Extract list of base expressions
-    bases = string.splitfields(basepart, ',')
+    bases = basepart.split(',')
 
     # Strip trailing '()' from each base expression
     for i in range(len(bases)):
@@ -185,8 +182,9 @@ def fixline(line):
             bases[i] = bases[i][x1:y1]
 
     # Join the bases back again and build the new line
-    basepart = string.joinfields(bases, ', ')
+    basepart = ', '.join(bases)
 
     return head + '(' + basepart + '):' + tail
 
-main()
+if __name__ == '__main__':
+    main()
