@@ -60,8 +60,8 @@ def fail(msg):
 # couldn't be opened
 def fopen(fname):
     try:
-        return open(fname, 'r')
-    except IOError, detail:
+        return open(fname, 'U')
+    except IOError as detail:
         return fail("couldn't open " + fname + ": " + str(detail))
 
 # open two files & spray the diff to stdout; return false iff a problem
@@ -74,7 +74,7 @@ def fcompare(f1name, f2name):
     a = f1.readlines(); f1.close()
     b = f2.readlines(); f2.close()
     for line in difflib.ndiff(a, b):
-        print line,
+        print(line, end=' ')
 
     return 1
 
@@ -85,7 +85,7 @@ def main(args):
     import getopt
     try:
         opts, args = getopt.getopt(args, "qr:")
-    except getopt.error, detail:
+    except getopt.error as detail:
         return fail(str(detail))
     noisy = 1
     qseen = rseen = 0
@@ -101,7 +101,7 @@ def main(args):
     if rseen:
         if args:
             return fail("no args allowed with -r option")
-        if whichfile in "12":
+        if whichfile in ("1", "2"):
             restore(whichfile)
             return 1
         return fail("-r value must be 1 or 2")
@@ -109,8 +109,8 @@ def main(args):
         return fail("need 2 filename args")
     f1name, f2name = args
     if noisy:
-        print '-:', f1name
-        print '+:', f2name
+        print('-:', f1name)
+        print('+:', f2name)
     return fcompare(f1name, f2name)
 
 # read ndiff output from stdin, and print file1 (which=='1') or
