@@ -4,7 +4,6 @@
 
 import os
 import sys
-import string
 import getopt
 
 def main():
@@ -12,42 +11,43 @@ def main():
     try:
         opts, args = getopt.getopt(sys.argv[1:], "t:")
         if not args:
-            raise getopt.error, "At least one file argument required"
-    except getopt.error, msg:
-        print msg
-        print "usage:", sys.argv[0], "[-t tabwidth] file ..."
+            raise getopt.error("At least one file argument required")
+    except getopt.error as msg:
+        print(msg)
+        print("usage:", sys.argv[0], "[-t tabwidth] file ...")
         return
     for optname, optvalue in opts:
         if optname == '-t':
             tabsize = int(optvalue)
 
-    for file in args:
-        process(file, tabsize)
+    for filename in args:
+        process(filename, tabsize)
 
-def process(file, tabsize):
+def process(filename, tabsize, verbose=True):
     try:
-        f = open(file)
+        f = open(filename)
         text = f.read()
         f.close()
-    except IOError, msg:
-        print "%s: I/O error: %s" % (`file`, str(msg))
+    except IOError as msg:
+        print("%r: I/O error: %s" % (filename, msg))
         return
-    newtext = string.expandtabs(text, tabsize)
+    newtext = text.expandtabs(tabsize)
     if newtext == text:
         return
-    backup = file + "~"
+    backup = filename + "~"
     try:
         os.unlink(backup)
     except os.error:
         pass
     try:
-        os.rename(file, backup)
+        os.rename(filename, backup)
     except os.error:
         pass
-    f = open(file, "w")
+    f = open(filename, "w")
     f.write(newtext)
     f.close()
-    print file
+    if verbose:
+        print(filename)
 
 if __name__ == '__main__':
     main()
